@@ -1,5 +1,3 @@
-context("reactivity")
-
 test_that("reactive and reactiveVal are functions", {
   expect_s3_class(reactive({1}), "function")
   expect_s3_class(reactiveVal(1), "function")
@@ -128,6 +126,23 @@ test_that("ReactiveValues", {
   expect_error(values$a <- 1)
 })
 
+test_that("reactiveValues keys are sorted", {
+  values <- reactiveValues(b=2, a=0)
+  values$C <- 13
+  values$A <- 0
+  values$c <- 3
+  values$B <- 12
+  # Setting an existing value shouldn't change order
+  values$a <- 1
+  values$A <- 11
+
+  expect_identical(isolate(names(values)), c("b", "a", "C", "A", "c", "B"))
+  expect_identical(
+    isolate(reactiveValuesToList(values)),
+    list(b=2, a=1, C=13, A=11, c=3, B=12)
+  )
+})
+
 test_that("reactiveValues() has useful print method", {
   verify_output(test_path("print-reactiveValues.txt"), {
     x <- reactiveValues(x = 1, y = 2, z = 3)
@@ -702,9 +717,9 @@ test_that("reactive() accepts injected quosures", {
   y <- quo(a)
   exp <- quo(!!y + 10)
   a <- 2
-  f <- inject(reactive(!! exp ))
+  ff <- inject(reactive(!! exp ))
   a <- 3
-  expect_identical(isolate(f()), 13)
+  expect_identical(isolate(ff()), 13)
 })
 
 test_that("observe() accepts injected quosures", {
@@ -889,7 +904,9 @@ test_that("Observers fire in consistent order across platforms", {
   })
 
   for (i in 1:20) {
-    v(isolate(v()) + 1); shiny:::flushReact()
+    suppressMessages({
+      v(isolate(v()) + 1); shiny:::flushReact()
+    })
   }
 
   expected_order <- list()
@@ -1155,7 +1172,7 @@ test_that("Flush completes even when errors occur", {
   vals$x <- 0
   suppress_stacktrace(
     # Errors in reactive are translated to warnings in observers by default
-    expect_warning(flushReact())
+    expect_warning(expect_warning(flushReact()))
   )
   # Both observers should run up until the reactive that errors
   expect_true(all(c(n11, n12, n21, n22) == c(2,1,2,1)))
@@ -1214,7 +1231,9 @@ test_that("event handling helpers take correct dependencies", {
   expect_equal(execCount(o2), 2)
 })
 
-run_debounce_throttle <- function(do_priming) {
+
+test_that("debounce/throttle work properly (with priming)", {
+  do_priming <- TRUE
   # Some of the CRAN test machines are heavily loaded and so the timing for
   # these tests isn't reliable. https://github.com/rstudio/shiny/pull/2789
   skip_on_cran()
@@ -1257,8 +1276,8 @@ run_debounce_throttle <- function(do_priming) {
     expect_identical(isolate(tr()), 0)
   }
 
-  # Pump timer and reactives for about 1.4 seconds
-  stopAt <- Sys.time() + 1.4
+  # Pump timer and reactives for about 1.3 seconds
+  stopAt <- Sys.time() + 1.3
   while (Sys.time() < stopAt) {
     timerCallbacks$executeElapsed()
     flushReact()
@@ -1289,13 +1308,85 @@ run_debounce_throttle <- function(do_priming) {
   isolate(expect_identical(rv$a, dr()))
   expect_identical(tr_fired, 4)
   isolate(expect_identical(rv$a, tr()))
-}
-
-test_that("debounce/throttle work properly (with priming)", {
-  run_debounce_throttle(TRUE)
 })
+
+# Identical to test block above, but with do_priming set to FALSE.
 test_that("debounce/throttle work properly (without priming)", {
-  run_debounce_throttle(FALSE)
+  do_priming <- FALSE
+  # Some of the CRAN test machines are heavily loaded and so the timing for
+  # these tests isn't reliable. https://github.com/rstudio/shiny/pull/2789
+  skip_on_cran()
+
+  # The changing of rv$a will be the (chatty) source of reactivity.
+  rv <- reactiveValues(a = 0)
+
+  # This observer will be what changes rv$a.
+  src <- observe({
+    invalidateLater(100)
+    rv$a <- isolate(rv$a) + 1
+  })
+  on.exit(src$destroy(), add = TRUE)
+
+  # Make a debounced reactive to test.
+  dr <- debounce(reactive(rv$a), 500)
+
+  # Make a throttled reactive to test.
+  tr <- throttle(reactive(rv$a), 500)
+
+  # Keep track of how often dr/tr are fired
+  dr_fired <- 0
+  dr_monitor <- observeEvent(dr(), {
+    dr_fired <<- dr_fired + 1
+  })
+  on.exit(dr_monitor$destroy(), add = TRUE)
+
+  tr_fired <- 0
+  tr_monitor <- observeEvent(tr(), {
+    tr_fired <<- tr_fired + 1
+  })
+  on.exit(tr_monitor$destroy(), add = TRUE)
+
+  # Starting values are both 0. Earlier I found that the tests behaved
+  # differently if I accessed the values of dr/tr before the first call to
+  # flushReact(). That bug was fixed, but to ensure that similar bugs don't
+  # appear undetected, we run this test with and without do_priming.
+  if (do_priming) {
+    expect_identical(isolate(dr()), 0)
+    expect_identical(isolate(tr()), 0)
+  }
+
+  # Pump timer and reactives for about 1.3 seconds
+  stopAt <- Sys.time() + 1.3
+  while (Sys.time() < stopAt) {
+    timerCallbacks$executeElapsed()
+    flushReact()
+    Sys.sleep(0.001)
+  }
+
+  # dr() should not have had time to fire, other than the initial run, since
+  # there haven't been long enough gaps between invalidations.
+  expect_identical(dr_fired, 1)
+  # The value of dr() should not have updated either.
+  expect_identical(isolate(dr()), 0)
+
+  # tr() however, has had time to fire multiple times and update its value.
+  expect_identical(tr_fired, 3)
+  expect_identical(isolate(tr()), 10)
+
+  # Now let some time pass without any more updates.
+  src$destroy() # No more updates
+  stopAt <- Sys.time() + 1
+  while (Sys.time() < stopAt) {
+    timerCallbacks$executeElapsed()
+    flushReact()
+    Sys.sleep(0.001)
+  }
+
+  # dr should've fired, and we should have converged on the right answer.
+  expect_identical(dr_fired, 2)
+  isolate(expect_identical(rv$a, dr()))
+  expect_identical(tr_fired, 4)
+  isolate(expect_identical(rv$a, tr()))
 })
 
 test_that("reactive domain works across async handlers", {
@@ -1605,3 +1696,41 @@ test_that("Reactive expression labels", {
     "hello"
   )
 })
+
+
+test_that("Contexts can be masked off", {
+  expect_error(
+    {
+      r <- reactiveVal()
+      isolate({
+        maskReactiveContext({
+          r()
+        })
+      })
+    },
+    regexp = "Operation not allowed without an active reactive context"
+  )
+})
+
+
+test_that("Contexts can be masked off via promise domains", {
+  r <- reactiveVal()
+  done <- FALSE
+  isolate({
+    maskReactiveContext({
+      promises::promise_resolve(NULL)$then(function(value) {
+        done <<- TRUE
+        expect_error(
+          {
+            r()
+          },
+          regexp = "Operation not allowed without an active reactive context"
+        )
+      })
+    })
+  })
+  while (!done) {
+    later::run_now(all=FALSE)
+  }
+})
+
